###############################################################################
#
# This perl module contains the configuration information for this parser.
# If you don't understand the format of this configuration file DON'T TOUCH IT.
#
###############################################################################
# Licensed Materials - Property of IBM
#
#  5724-T01 5724-S98
#
# (c) Copyright IBM Corp. 2003, 2008. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
###############################################################################

package EngineConfig;

@ISA    = qw(Exporter);
@EXPORT = qw(engine_config);

#use diagnostics;
use strict;
use AudLog;
use Exporter ();
use Huawei_BSS_Counter_Map;

use Data::Dumper;

my $bss_map = Huawei_BSS_Counter_Map::return_bss_ref();
my ($DO_BSS_MAPPING) = 1;    # 1 = perform mapping, 0 = otherwise

#-------------------------------------------------------------------------------

##
## Maps raw data counter name to GW Counter Name
sub mapBSSCounterID {
    my ( $entity_name, $object_type ) = @_;
    my $value;

    # only a matter of
    #      - look for the key in the HASH and return the value
    #      - else, return back the key

    #LogMess("DEBUG: entity_name=$entity_name, object_type=$object_type\n",5);

    #LogMess("DEBUG: BSS_MAP -> \n" . Dumper($bss_map), 5);

    if ( exists $bss_map->{$object_type}{$entity_name} ) {
        $value = $bss_map->{$object_type}{$entity_name};
    }
    #elsif ($entity_name =~ /^53687\d+/ or $entity_name =~ /^58720\d+/) {
	elsif ($entity_name =~ /^5368\d+/ or $entity_name =~ /^58720\d+/) {
        $value = qq{c} . $entity_name;
    }
    else {
        $value = $entity_name;
    }

    #LogMess("DEBUG: value=$value\n",5);

    return $value;
}

##
## Extracts and reformats STARTDATE and STARTTIME from Result_time
sub get_date_time
{
    my ( $h_ref, $d_ref ) = @_;

    if ($d_ref->{Result_Time}=~/\d{4}\-\d{1,2}\-\d{1,2}\_\d{1,2}\:\d{2}/) {
        my $datestring = $d_ref->{Result_Time};
        $datestring =~ tr/\/,\-,\:/_/;
        my($yyyy, $mm, $dd, $hh, $mi) = split/_/,$datestring;

        # Preceed with zeroes
        $mm = sprintf("%02d", $mm);
        $dd = sprintf("%02d", $dd);
        $hh = sprintf("%02d", $hh);

        my ($STARTDATE, $STARTTIME);
        $STARTDATE = "$yyyy\-$mm\-$dd";
        $STARTTIME = "$hh:$mi";

        if (exists $h_ref->{DATE} && $h_ref->{DATE} eq "0000-00-00") {
            delete $h_ref->{Result_Time};
            $h_ref->{DATE} = "$STARTDATE";
        }

        # remove column Datetime, and add column STARTDATE, STARTTIME;
        delete $d_ref->{Result_Time};
        $d_ref->{STARTDATE} = $STARTDATE;
        $d_ref->{STARTTIME} = $STARTTIME;
    }
    elsif( $d_ref->{Result_Time} =~ /(\d{1,2})\/(\d{1,2})\/(\d{4})\_(\d{1,2})\:(\d{2})/ ) {
        my ($mm, $dd, $yyyy, $hh, $mi) = ( $1, $2, $3, $4, $5 );

        # Preceed with zeroes
        $mm = sprintf("%02d", $mm);
        $dd = sprintf("%02d", $dd);
        $hh = sprintf("%02d", $hh);

        my ($STARTDATE, $STARTTIME);
        $STARTDATE = "$yyyy\-$mm\-$dd";
        $STARTTIME = "$hh:$mi";

        if (exists $h_ref->{DATE} && $h_ref->{DATE} eq "0000-00-00") {
            delete $h_ref->{Result_Time};
            $h_ref->{DATE} = "$STARTDATE";
        }

        # remove column Datetime, and add column STARTDATE, STARTTIME;
        delete $d_ref->{Result_Time};
        $d_ref->{STARTDATE} = $STARTDATE;
        $d_ref->{STARTTIME} = $STARTTIME;
    }
}

##
## Removes invalid characters from Counter Name and Value
sub process_counters
{
    my ($d_ref, $block_prefix, $block_name) = @_;
    my ($oldkeys, $newkeys, $mapvalue);


    foreach $newkeys (keys %$d_ref) {

        $oldkeys = $newkeys;

        # removing _Ind* cells available at the end of the CSV file
        if ($newkeys =~ /^_Ind+/) {
            delete($d_ref->{$newkeys});
        }
        elsif ($newkeys =~ /^_CellNo/) {

            $newkeys =~ s/^_+//;
            $d_ref->{$newkeys} = $d_ref->{$oldkeys};
            $d_ref->{$newkeys} =~ s/^_+//;
            delete($d_ref->{$oldkeys});
        }
        else{

            # replacing invalid characters in the counter name
            $oldkeys = $newkeys;
            if ($newkeys =~ tr/[\(,\),\%,\/,\-]\:\.\=\<\>\?\~\t\r/_/) {
                $d_ref->{$newkeys} = $d_ref->{$oldkeys};
                delete($d_ref->{$oldkeys});
            }


            # removing the occurence of '_' at the end of the counter name
            $oldkeys = $newkeys;
            if ($newkeys =~ s/_+$//) {
                $d_ref->{$newkeys} = $d_ref->{$oldkeys};
                delete($d_ref->{$oldkeys});
            }


            # removing the occurence of '_' at the start of the counter name
            $oldkeys = $newkeys;
            if ($newkeys =~ s/^_+//) {
                $d_ref->{$newkeys} = $d_ref->{$oldkeys};
                delete($d_ref->{$oldkeys});
            }


            # replacing the occurence of 2x or more '_' into just one (e.g '__', '___')
            $oldkeys = $newkeys;
            if ($newkeys =~ s/_{2,}/_/g) {
                $d_ref->{$newkeys} = $d_ref->{$oldkeys};
                delete($d_ref->{$oldkeys});
            }


            # Trim leading and trailing underscores from data
            $d_ref->{$newkeys} =~ s/_+$//;
            $d_ref->{$newkeys} =~ s/^_+//;


            # Remove tabs and return from data
            $d_ref->{$newkeys} =~ s/[\t\r]//g;
        }

        #  Processing the data -> doing the mapping based on the $DO_MAPPING flag
        if ($DO_BSS_MAPPING) {

            $mapvalue = mapBSSCounterID($newkeys, $block_prefix . $block_name);

            if ($mapvalue ne $newkeys) {

                $oldkeys = $newkeys;
                $newkeys = $mapvalue;

                $d_ref->{$newkeys} = $d_ref->{$oldkeys};

                delete($d_ref->{$oldkeys});
            }
        }
    }
}

#-------------------------------------------------------------------------------
# Modify/Add rules here
#-------------------------------------------------------------------------------
my @rules = (

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : A_Interface
    #-----------------------------------------
    {
        RULE_TYPE              => 'GENERIC_CSV_GATEWAY',
        RULE_DESC              => 'Generic CSV Gateway For GSM/BSC Data - A_Interface',
        INPUT_FILE_DESCRIPTION => [
            '^pmresult_536870914_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870915_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870916_\d{2}_\d{12}_\d{12}.csv$'
        ],
        INPUT_DIR_DEPTH            => 3,                ## Changefrom 0 to 3 to pick up NETWORK & REGION ID
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => ',',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).csv',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.csv$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.csv$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.csv$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.csv$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },

        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("A-Interface: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );

            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the BSC_Id from <BSC_Id>/BSCNE:HBSC_1_Pleven
            if ( $d_ref->{Object_Name} =~ /(.+)\/BSCNE:.+/ ) {
                $d_ref->{BSC_Id} = $1;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : BS
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GSM/BSC Data - BS',
        INPUT_FILE_DESCRIPTION     => [
            '^pmresult_536870931_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870952_\d{2}_\d{12}_\d{12}.csv$'
        ],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => ',',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).csv',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.csv$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.csv$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.csv$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.csv$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("BS: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );

            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the BSC_Id and BTSSiteManager from format <BSC_Id>/BTSSiteManager:<BTSSiteManager>
            if ( $d_ref->{Object_Name} =~ /(.+)\/BTSSiteManager:(.+)/ ) {
                $d_ref->{BSC_Id} = $1;
                $d_ref->{BTSSiteManager} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : BSC
    #-----------------------------------------
    {
        RULE_TYPE              => 'GENERIC_CSV_GATEWAY',
        RULE_DESC              => 'Generic CSV Gateway For GSM/BSC Data - BSC',
        INPUT_FILE_DESCRIPTION => [
            '^pmresult_536870912_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870913_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870918_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870919_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870920_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870964_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870965_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870974_\d{2}_\d{12}_\d{12}.csv$',
        ],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => ',',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).csv',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.csv$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.csv$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.csv$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.csv$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("BSC: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the BSC_Id name from format <BSC_Id>/BSCNE:HBSC_1_Pleven
            if ( $d_ref->{Object_Name} =~ /(.+)\/BSCNE:.+/ ) {
                $d_ref->{BSC_Id} = $1;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : Cell
    #-----------------------------------------
    {
        RULE_TYPE              => 'GENERIC_CSV_GATEWAY',
        RULE_DESC              => 'Generic CSV Gateway For GSM/BSC Data - Cell',
        INPUT_FILE_DESCRIPTION => [
            '^pmresult_536870922_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870923_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870924_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870925_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870926_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870927_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870928_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870929_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870930_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870932_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870933_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870936_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870937_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870947_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870948_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870949_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870950_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870951_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870966_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870967_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870968_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870971_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870972_\d{2}_\d{12}_\d{12}.csv$',
        ],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => ',',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).csv',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.csv$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.csv$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.csv$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.csv$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },

        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("Cell: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );

            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the attributes
            ## Case 1: BSCBD01/Cell:Label=BD051GB,_MODUNO=5,_CELLLNUM=34/ThresholdRate:5/ThresholdLevel:5/IndexNO:1
            if ( $d_ref->{Object_Name} =~ /(.+)\/Cell:Label=(.+)(\w),_MODUNO=(\d+),_CELLLNUM=(\d+)\/ThresholdRate:(\d+)\/ThresholdLevel:(\d+)\/IndexNO:(\d+)/ ) {

                # Remove hyphen and append MODUNO and CELLNUM
                $d_ref->{Cell}  = $2 . $3;
                $d_ref->{Cell}  =~ s/-//g;
                $d_ref->{Cell} .= "-" . $4 . "-" . $5;

                $d_ref->{BSC_Id}         = $1;
                $d_ref->{BTSSiteManager} = $2;
                $d_ref->{ThresholdRate}  = $6;
                $d_ref->{ThresholdLevel} = $7;
                $d_ref->{IndexNO}        = $8;
            }
            ## Case 2: BSCBD01/Cell:Label=BD051GB,_MODUNO=5,_CELLLNUM=34
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:Label=(.+)(\w),_MODUNO=(\d+),_CELLLNUM=(\d+)/ ) {

                # Remove hyphen and append MODUNO and CELLNUM
                $d_ref->{Cell}  = $2 . $3;
                $d_ref->{Cell}  =~ s/-//g;
                $d_ref->{Cell} .= "-" . $4 . "-" . $5;

                $d_ref->{BSC_Id}         = $1;
                $d_ref->{BTSSiteManager} = $2;
            }
            ## Case 3: BPLM1/Cell:4686_Muara_Rupit-1/ThresholdRate:10/ThresholdLevel:15
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)\/ThresholdRate:(\d+)\/ThresholdLevel:(\d+)/ ) {

                $d_ref->{BSC_Id}         = $1;
                $d_ref->{BTSSiteManager} = $2;
                $d_ref->{Cell}           = $2;
                $d_ref->{ThresholdRate}  = $3;
                $d_ref->{ThresholdLevel} = $4;
                
                # Remove hyphen
                $d_ref->{Cell} =~ s/-//g;
            }
            ## Case 4: HBSC_1_Pleven/Cell:VT5251-3
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)-(.+)/ ) {

                $d_ref->{BSC_Id} = $1;
                $d_ref->{BTSSiteManager} = $2;

                # Remove hyphen then append number
                $d_ref->{Cell}  = $2;
                $d_ref->{Cell}  =~ s/-//g;
                $d_ref->{Cell} .= $3;
            }
            ## Case 5: BMDN2/Cell:MC502_SBaruMedan
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)/ ) {

                $d_ref->{BSC_Id}         = $1;
                $d_ref->{BTSSiteManager} = $2;
                $d_ref->{Cell}           = $3;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : LAPD
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GSM/BSC Data - LAPD',
        INPUT_FILE_DESCRIPTION     => ['^pmresult_536870943_\d{2}_\d{12}_\d{12}.csv$'],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => ',',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).csv',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.csv$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.csv$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.csv$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.csv$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("LAPD: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );

            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the LAPDLink id from format <BSC_Id>/LAPDLink:<LAPDLink>
            # Extracting the BSC_Id name from format <BSC_Id>/LAPDLink:<LAPDLink>
            if ( $d_ref->{Object_Name} =~ /(.+)\/LAPDLink:(.+)/ ) {
                $d_ref->{BSC_Id}   = $1;
                $d_ref->{LAPDLink} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : Neighbour
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GSM/BSC Data - Neighbour',
        INPUT_FILE_DESCRIPTION     => [
            '^pmresult_536870934_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870935_\d{2}_\d{12}_\d{12}.csv$'
        ],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => ',',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).csv',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.csv$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.csv$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.csv$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.csv$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("Neighbour: BSC: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );

            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the Cell, BSC_Id, and AdjCell
            ## Case 1: BSCSB01/Cell:Label=ST067GB,_MODUNO=8,_CELLLNUM=8/AdjCell:ST007GB or
            ##         BSCSB01/Cell:Label=ST067GB,_MODUNO=8,_CELLLNUM=8/AdjCell:284033f13ecc667 (hex)
            if ( $d_ref->{Object_Name} =~ /(.+)\/Cell:Label=(.+),_?MODUNO=(\d+),_?CELLLNUM=(\d+)[,_|\/]+AdjCell:(.+)/ ) {

                if ( $block_name eq 'M536870934' ) {

                    # Remove hyphen and append MODUNO and CELLNUM
                    $d_ref->{Cell}  = $2;
                    $d_ref->{Cell}  =~ s/-//g;
                    $d_ref->{Cell} .= "-" . $3. "-" . $4;

                    $d_ref->{BSC_Id}  = $1;
                    $d_ref->{AdjCell} = $5;
                }
                elsif ( $block_name eq 'M536870935' ) {

                    # Remove hyphen and append MODUNO and CELLNUM
                    $d_ref->{Cell}  = $2;
                    $d_ref->{Cell}  =~ s/-//g;
                    $d_ref->{Cell} .= "-" . $3. "-" . $4;

                    $d_ref->{BSC_Id} = $1;
                    $d_ref->{AdjCell} = $5;

                    # Handling AdjCell # in hexa for block M536870935
                    my $myAdjCell_Hexa = substr( $5, -4 );
                    $d_ref->{AdjCell} = hex($myAdjCell_Hexa);
                }
                else {
                    LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
                }
            }
            ## Case 2: HBSC_1_Pleven/Cell:VT5251-3/AdjCell:VT5048-1
            ## CR valnt00016762, 09 Apr 2008
            ## AdjCell should be without prefix characters (cf. AdjCell 50481)
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)-(\d+)\/AdjCell:(\w+)-(\d+)/ ) {
                ## CR valnt00007899,02 Oct 2007
                ## AdjCell should also be without "-" (cf. AdjCell VT50481)
                $d_ref->{BSC_Id} = $1;
                $d_ref->{Cell}   = $2 . $3;

                ## CR valnt00007899,02 Oct 2007
                ## AdjCell should also be without "-" (cf. AdjCell VT50481)
                $d_ref->{AdjCell} = $4 . $5;
            }
            ## Case 3: HBSC_1_Pleven/Cell:VT5251-3/AdjCell:284033f13ecc667 (hex)
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)-(\d+)\/AdjCell:(.+)/ ) {
                $d_ref->{BSC_Id} = $1;
                $d_ref->{Cell}   = $2 . $3;

                # Handling AdjCell # in hexa for block M536870935
                my $myAdjCell_Hexa = substr( $4, -4 );
                $d_ref->{AdjCell} = hex($myAdjCell_Hexa);
            }
            ## Case 4: BMDN2/Cell:MC502_SBaruMedan/AdjCell:3070_Aurmedan-3
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)\/AdjCell:(.+)-(\d+)/ ) {
                $d_ref->{BSC_Id}  = $1;
                $d_ref->{Cell}    = $2;
                $d_ref->{AdjCell} = $3 . $4;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : Processor
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GSM/BSC Data - Processor',
        INPUT_FILE_DESCRIPTION     => ['^pmresult_536870917_\d{2}_\d{12}_\d{12}.csv$'],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => ',',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).csv',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.csv$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.csv$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.csv$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.csv$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("Processor: BSC: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );

            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the BSC_Id, and Module from format <BSC_Id>/Module:<Module>
            if ( $d_ref->{Object_Name} =~ /(.+)\/Module:(.+)/ ) {
                $d_ref->{BSC_Id} = $1;
                $d_ref->{Module} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : Signalling_Link
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GSM/BSC Data - Signalling_Link',
        INPUT_FILE_DESCRIPTION     => ['^pmresult_536870921_\d{2}_\d{12}_\d{12}.csv$'],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => ',',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).csv',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.csv$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.csv$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.csv$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.csv$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("Signalling Link: BSC: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );

            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the BSC_Id, SignLink, SignLinkset from format <BSC_Id>/SignLink:<SignLink>
            ## Case 1: HBSC_1_Pleven/SignLink:MSC-2-8
            if ( $d_ref->{Object_Name} =~ /(.+)\/SignLink:(\w+-\w+)(-\w+)/ ) {
                $d_ref->{BSC_Id}      = $1;
                $d_ref->{SignLink}    = $2 . $3;
                $d_ref->{SignLinkset} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : TC_Board
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GSM/BSC Data - TC_Board',
        INPUT_FILE_DESCRIPTION     => [
            '^pmresult_536870957_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870970_\d{2}_\d{12}_\d{12}.csv$'
        ],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => ',',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).csv',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.csv$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.csv$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.csv$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.csv$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("TC_BOARD: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );

            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the BSC_Id, Module, Frame, Board from format <BSC_Id>/Module:<Module>/Frame:<Frame>/Board:<Board>
            if ( $d_ref->{Object_Name} =~ /(.+)\/Module:(.+)\/Frame:(.+)\/Board:(.+)/ ) {
                $d_ref->{BSC_Id} = $1;
                $d_ref->{Module} = $2;
                $d_ref->{Frame}  = $3;
                $d_ref->{Board}  = $4;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : TRX
    #-----------------------------------------
    {
        RULE_TYPE              => 'GENERIC_CSV_GATEWAY',
        RULE_DESC              => 'Generic CSV Gateway For GSM/BSC Data - TRX',
        INPUT_FILE_DESCRIPTION => [
            '^pmresult_526870942_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870938_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870939_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870940_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870941_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870942_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870944_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870945_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870946_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870953_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870954_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870955_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870956_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_536870969_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_536870973_\d{2}_\d{12}_\d{12}.csv$'
        ],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => ',',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).csv',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.csv$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.csv$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.csv$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.csv$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("TRX: d_ref -> \n" . Dumper($d_ref), 5);

            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );

            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the BSC_Id, Cell, Trx
            ## Case 1: BSCSB01/Cell:Label=ST067GC,_MODUNO=8,_CELLLNUM=9/Trx:TRX-11
            if ( $d_ref->{Object_Name} =~ /(.+)\/Cell:Label=(.+),_?MODUNO=(\d+),_?CELLLNUM=(\d+)\/Trx:(.+)/ ) {

                # Remove hyphen then append MODUNO and CELLNUM
                $d_ref->{Cell}  = $2;
                $d_ref->{Cell}  =~ s/-//g;
                $d_ref->{Cell} .= "-" . $3 . "-" . $4;

                $d_ref->{BSC_Id} = $1;
                $d_ref->{Trx}  = $5;
            }
            ## Case 2: HBSC_1_Pleven/Cell:VT5378-2/Trx:TRX-4
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)-(.+)\/Trx:(.+)/ ) {

                ## CR valnt00007899,02 Oct 2007
                ## Cell should append the numeric number after the  "-" (cf. Cell:Test_Site-3 to be Cell Test_Site3)
                ## old code: $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)-.+\/Trx:(.+)/;
                ## 26 Oct 2007
                ## Fixing object name with BPKB1/Cell:4402_Nangka-3(BSC)/Trx:TRX-9
                ## \d+ would not match 3(BSC), hence change it to .+
                ## old code: $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)-(\d+)\/Trx:(.+)/;

                $d_ref->{BSC_Id} = $1;

                ## CR valnt00007899,02 Oct 2007
                ## Cell should append the numeric number after the  "-" (cf. Cell:Test_Site-3 to be Cell Test_Site3)
                $d_ref->{Cell} = $2 . $3;
                $d_ref->{Trx}  = $4;
            }
            ## Case 3: BMDN2/Cell:MC502_SBaruMedan/Trx:TRX-1
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)\/Trx:(.+)/ ) {

                $d_ref->{BSC_Id} = $1;
                $d_ref->{Cell}   = $2;
                $d_ref->{Trx}    = $3;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GPRS Files (with counter name) - BSC
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GPRS/PCU DATA - BSC',
        INPUT_FILE_DESCRIPTION     => ['^pmresult_587203160_\d{2}_\d{12}_\d{12}.csv$'],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => ',',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).csv',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.csv$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.csv$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.csv$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.csv$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;    ## 02 Nov 2007, declare $block_name

            #LogMess("PCU-BSC: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};                     ## 02 Nov 2007
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );

            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Process the counter name and data
            process_counters($d_ref, "PCU_", $block_name);


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the PCU_Id, PCUBSC from format <PCU_Id>/PCUBSC:<PCUBSC>
            if ( $d_ref->{Object_Name} =~ /(.+)\/PCUBSC:(.+)/ ) {
                $d_ref->{PCU_Id} = $1;
                $d_ref->{PCUBSC} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GPRS Files (with counter name) - CELL
    #-----------------------------------------
    {
        RULE_TYPE              => 'GENERIC_CSV_GATEWAY',
        RULE_DESC              => 'Generic CSV Gateway For GPRS/PCU DATA - CELL',
        INPUT_FILE_DESCRIPTION => [
            '^pmresult_587203010_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_587202560_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_587202580_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_587202600_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_587202620_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_587202640_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_587202660_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_587202690_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_587202710_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_587202740_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_587202770_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_587202800_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_587202830_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_587202860_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_587202890_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_587202920_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_587202940_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_587202960_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_587202970_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_587202990_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_587203040_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_587203070_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_587203110_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_587204560_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_587204570_\d{2}_\d{12}_\d{12}.csv$', '^pmresult_587204580_\d{2}_\d{12}_\d{12}.csv$',
            '^pmresult_587204590_\d{2}_\d{12}_\d{12}.csv$'
        ],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => ',',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).csv',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.csv$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.csv$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.csv$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.csv$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;    ## 02 Nov 2007, declare $block_name

            #LogMess("PCU-Cell: d_ref -> \n" . Dumper($d_ref), 5);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {
                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;
            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the PCU_Id, PCUCell, PCUCell_name, BTSSiteManager
            ## Case 1: PCU2/PCUCell:syl_x0043_1
            if ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(\w+)_(\D+\d+)_(\d+)/ ) {

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $3 . $4;
                $d_ref->{PCUCell_name}   = $2 . "_" . $3 . "_" . $4;
                $d_ref->{BTSSiteManager} = $3;

            }
            ## Case 2: PCUMKS1/PCUCell:3747_antang-1 or PCUPTK1/PCUCell:3408+sambas-1 or PCUMKS1/PCUCell:3747antang-1
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(\d{4})[-+_]?(.+)-(\d{1})/ ) {

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $2 . $4;
                $d_ref->{PCUCell_name}   = $3 . "-" . $4;
                $d_ref->{BTSSiteManager} = $2;

            }
            ## Case 3: PCUMKS1/PCUCell:37463_sengkang or PCUBLP1/PCUCell:39712kuriang
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(\d{4})(\d{1})[-+_]?(.+)/ ) {

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $2 . $3;
                $d_ref->{PCUCell_name}   = $4 . "-" . $3;
                $d_ref->{BTSSiteManager} = $2;

            }
            ## Case 4: PCUMKS1/PCUCell:pc549_mtcmakassar-1
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(\w+)[-+_](\w+)-(\d+)/ ) {

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $2 . $4;
                $d_ref->{PCUCell_name}   = $3 . "-" . $4;
                $d_ref->{BTSSiteManager} = $2;

            }
            ## Case 5: PCUMKS1/PCUCell:pc197_gtc_tj_bunga
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(.+\d{1})[-+_](.+)/ ) {

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $2;
                $d_ref->{PCUCell_name}   = $3;
                $d_ref->{BTSSiteManager} = $2;

            }
            ## Case 6: PCU2/PCUCell:vn45423=huawei_kaolinovo
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(.+)(\d{1})=(.+)/ ){

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $2 . $3;
                $d_ref->{PCUCell_name}   = $4 . "-" . $3;
                $d_ref->{BTSSiteManager} = $2;

            }
            ## Case 7: PCU4/PCUCell:tbtshu_1 or PCU_URA/PCUCell:urfeg-1
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(\w+)[_-](\d+)/ ) {

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $2 . $3;
                $d_ref->{PCUCell_name}   = $2 . $3;
                $d_ref->{BTSSiteManager} = $2;

            }
            ## Case 8: PCU1/PCUCell:test
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(.+)/ ) {

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $2;
                $d_ref->{PCUCell_name}   = $2;
                $d_ref->{BTSSiteManager} = $2;

            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


						if ( !defined($d_ref->{PCU_Id}) or $d_ref->{PCU_Id} eq "" or
						     !defined($d_ref->{PCUCell}) or $d_ref->{PCUCell} eq "" or
						     !defined($d_ref->{PCUCell_name}) or $d_ref->{PCUCell_name} eq "" or
						     !defined($d_ref->{BTSSiteManager}) or $d_ref->{BTSSiteManager} eq "" ) {
                LogMess("Invalid objects for PCU DATA - CELL!", 5);
            }


            # Ignore record if PCU Id is not valid
            if ( $d_ref->{PCU_Id} =~ /^[\s_\/\\]$/ ) {
                LogMess( "{PCU_Id} is equal to blank or underscore. The related block is removed." );
                $$blkname_ref = "";
                return;
            }


            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "PCU_", $block_name);


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }


            return;
        }
    },

    #-----------------------------------------
    # Huawei BSS GPRS Files (with counter name) - LAPD
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GPRS/PCU DATA - LAPD',
        INPUT_FILE_DESCRIPTION     => ['^pmresult_587203180_\d{2}_\d{12}_\d{12}.csv$'],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => ',',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).csv',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.csv$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.csv$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.csv$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.csv$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;    ## 02 Nov 2007, declare $block_name

            #LogMess("PCU-LAPD: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};                     ## 02 Nov 2007
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );

            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Process the counter name and data
            process_counters($d_ref, "PCU_", $block_name);


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the PCU_Id, PCULAPD from format <PCU_Id>/PCULAPD:<PCULAPD>
            if ( $d_ref->{Object_Name} =~ /(.+)\/PCULAPD:(.+)/ ) {
                $d_ref->{PCU_Id}  = $1;
                $d_ref->{PCULAPD} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GPRS Files (with counter name) - NSVC
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GPRS/PCU DATA - NSVC',
        INPUT_FILE_DESCRIPTION     => ['^pmresult_587203210_\d{2}_\d{12}_\d{12}.csv$'],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => ',',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).csv',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.csv$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.csv$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.csv$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.csv$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;    ## 02 Nov 2007, declare $block_name

            #LogMess("PCU-NSVC: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};                     ## 02 Nov 2007
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );

            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Process the counter name and data
            process_counters($d_ref, "PCU_", $block_name);


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the PCU_Id, PCULAPD from format <PCU_Id>/PCUNSVC:<PCUNSVC>
            if ( $d_ref->{Object_Name} =~ /(.+)\/PCUNSVC:(.+)/ ) {
                $d_ref->{PCU_Id}  = $1;
                $d_ref->{PCUNSVC} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GPRS Files (with counter name) - Processor
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GPRS/PCU DATA - Processor',
        INPUT_FILE_DESCRIPTION     => ['^pmresult_587203240_\d{2}_\d{12}_\d{12}.csv$'],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => ',',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).csv',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.csv$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.csv$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.csv$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.csv$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },

        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;    ## 02 Nov 2007, declare $block_name

            #LogMess("PCU-Processor: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};                     ## 02 Nov 2007
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );

            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Process the counter name and data
            process_counters($d_ref, "PCU_", $block_name);


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the PCU_Id, PCUBoard from format <PCU_Id>/PCUBoard:<PCUBoard>
            if ( $d_ref->{Object_Name} =~ /(.+)\/PCUBoard:(.+)/ ) {
                $d_ref->{PCU_Id}   = $1;
                $d_ref->{PCUBoard} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },


##############################################
#
# For Handling tab delimited, .txt raw file
#
##############################################


    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : A_Interface (txt)
    #-----------------------------------------
    {
        RULE_TYPE              => 'GENERIC_CSV_GATEWAY',
        RULE_DESC              => 'Generic CSV Gateway For GSM/BSC Data - A_Interface',
        INPUT_FILE_DESCRIPTION => [
            '^pmresult_536870914_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870915_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870916_\d{2}_\d{12}_\d{12}.txt$'
        ],
        INPUT_DIR_DEPTH            => 3,                ## Changefrom 0 to 3 to pick up NETWORK & REGION ID
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => '\t',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).txt',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.txt$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.txt$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.txt$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.txt$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },

        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("A-Interface: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the BSC_Id from <BSC_Id>/BSCNE:HBSC_1_Pleven
            if ( $d_ref->{Object_Name} =~ /(.+)\/BSCNE:.+/ ) {
                $d_ref->{BSC_Id} = $1;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : BS (txt)
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GSM/BSC Data - BS',
        INPUT_FILE_DESCRIPTION     => [
            '^pmresult_536870931_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870952_\d{2}_\d{12}_\d{12}.txt$'
        ],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => '\t',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).txt',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.txt$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.txt$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.txt$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.txt$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("BS: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the BSC_Id and BTSSiteManager from format <BSC_Id>/BTSSiteManager:<BTSSiteManager>
            if ( $d_ref->{Object_Name} =~ /(.+)\/BTSSiteManager:(.+)/ ) {
                $d_ref->{BSC_Id} = $1;
                $d_ref->{BTSSiteManager} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : BSC (txt)
    #-----------------------------------------
    {
        RULE_TYPE              => 'GENERIC_CSV_GATEWAY',
        RULE_DESC              => 'Generic CSV Gateway For GSM/BSC Data - BSC',
        INPUT_FILE_DESCRIPTION => [
            '^pmresult_536870912_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870913_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870918_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870919_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870920_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870964_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870965_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870974_\d{2}_\d{12}_\d{12}.txt$',
        ],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => '\t',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).txt',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.txt$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.txt$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.txt$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.txt$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("BSC: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the BSC_Id name from format <BSC_Id>/BSCNE:HBSC_1_Pleven
            if ( $d_ref->{Object_Name} =~ /(.+)\/BSCNE:.+/ ) {
                $d_ref->{BSC_Id} = $1;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : Cell (txt)
    #-----------------------------------------
    {
        RULE_TYPE              => 'GENERIC_CSV_GATEWAY',
        RULE_DESC              => 'Generic CSV Gateway For GSM/BSC Data - Cell',
        INPUT_FILE_DESCRIPTION => [
            '^pmresult_536870922_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870923_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870924_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870925_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870926_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870927_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870928_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870929_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870930_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870932_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870933_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870936_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870937_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870947_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870948_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870949_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870950_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870951_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870966_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870967_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870968_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870971_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870972_\d{2}_\d{12}_\d{12}.txt$',
        ],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => '\t',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).txt',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.txt$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.txt$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.txt$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.txt$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },

        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("Cell: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the attributes
            ## Case 1: BSCBD01/Cell:Label=BD051GB,_MODUNO=5,_CELLLNUM=34/ThresholdRate:5/ThresholdLevel:5/IndexNO:1
            if ( $d_ref->{Object_Name} =~ /(.+)\/Cell:Label=(.+)(\w),_MODUNO=(\d+),_CELLLNUM=(\d+)\/ThresholdRate:(\d+)\/ThresholdLevel:(\d+)\/IndexNO:(\d+)/ ) {

                # Remove hyphen and append MODUNO and CELLNUM
                $d_ref->{Cell}  = $2 . $3;
                $d_ref->{Cell}  =~ s/-//g;
                $d_ref->{Cell} .= "-" . $4 . "-" . $5;

                $d_ref->{BSC_Id}         = $1;
                $d_ref->{BTSSiteManager} = $2;
                $d_ref->{ThresholdRate}  = $6;
                $d_ref->{ThresholdLevel} = $7;
                $d_ref->{IndexNO}        = $8;
            }
            ## Case 2: BSCBD01/Cell:Label=BD051GB,_MODUNO=5,_CELLLNUM=34
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:Label=(.+)(\w),_MODUNO=(\d+),_CELLLNUM=(\d+)/ ) {

                # Remove hyphen and append MODUNO and CELLNUM
                $d_ref->{Cell}  = $2 . $3;
                $d_ref->{Cell}  =~ s/-//g;
                $d_ref->{Cell} .= "-" . $4 . "-" . $5;

                $d_ref->{BSC_Id}         = $1;
                $d_ref->{BTSSiteManager} = $2;
            }
            ## Case 3: BPLM1/Cell:4686_Muara_Rupit-1/ThresholdRate:10/ThresholdLevel:15
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)\/ThresholdRate:(\d+)\/ThresholdLevel:(\d+)/ ) {

                $d_ref->{BSC_Id}         = $1;
                $d_ref->{BTSSiteManager} = $2;
                $d_ref->{Cell}           = $2;
                $d_ref->{ThresholdRate}  = $3;
                $d_ref->{ThresholdLevel} = $4;
                
                # Remove hyphen
                $d_ref->{Cell} =~ s/-//g;
            }
            ## Case 4: HBSC_1_Pleven/Cell:VT5251-3
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)-(.+)/ ) {

                $d_ref->{BSC_Id} = $1;
                $d_ref->{BTSSiteManager} = $2;

                # Remove hyphen then append number
                $d_ref->{Cell}  = $2;
                $d_ref->{Cell}  =~ s/-//g;
                $d_ref->{Cell} .= $3;
            }
            ## Case 5: BMDN2/Cell:MC502_SBaruMedan
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)/ ) {

                $d_ref->{BSC_Id}         = $1;
                $d_ref->{BTSSiteManager} = $2;
                $d_ref->{Cell}           = $3;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : LAPD (txt)
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GSM/BSC Data - LAPD',
        INPUT_FILE_DESCRIPTION     => ['^pmresult_536870943_\d{2}_\d{12}_\d{12}.txt$'],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => '\t',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).txt',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.txt$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.txt$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.txt$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.txt$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("LAPD: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the LAPDLink id from format <BSC_Id>/LAPDLink:<LAPDLink>
            # Extracting the BSC_Id name from format <BSC_Id>/LAPDLink:<LAPDLink>
            if ( $d_ref->{Object_Name} =~ /(.+)\/LAPDLink:(.+)/ ) {
                $d_ref->{BSC_Id}   = $1;
                $d_ref->{LAPDLink} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : Neighbour (txt)
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GSM/BSC Data - Neighbour',
        INPUT_FILE_DESCRIPTION     => [
            '^pmresult_536870934_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870935_\d{2}_\d{12}_\d{12}.txt$'
        ],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => '\t',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).txt',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.txt$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.txt$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.txt$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.txt$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("Neighbour: BSC: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the Cell, BSC_Id, and AdjCell
            ## Case 1: BSCSB01/Cell:Label=ST067GB,_MODUNO=8,_CELLLNUM=8/AdjCell:ST007GB or
            ##         BSCSB01/Cell:Label=ST067GB,_MODUNO=8,_CELLLNUM=8/AdjCell:284033f13ecc667 (hex)
            if ( $d_ref->{Object_Name} =~ /(.+)\/Cell:Label=(.+),_?MODUNO=(\d+),_?CELLLNUM=(\d+)[,_|\/]+AdjCell:(.+)/ ) {

                if ( $block_name eq 'M536870934' ) {

                    # Remove hyphen and append MODUNO and CELLNUM
                    $d_ref->{Cell}  = $2;
                    $d_ref->{Cell}  =~ s/-//g;
                    $d_ref->{Cell} .= "-" . $3. "-" . $4;

                    $d_ref->{BSC_Id}  = $1;
                    $d_ref->{AdjCell} = $5;
                }
                elsif ( $block_name eq 'M536870935' ) {

                    # Remove hyphen and append MODUNO and CELLNUM
                    $d_ref->{Cell}  = $2;
                    $d_ref->{Cell}  =~ s/-//g;
                    $d_ref->{Cell} .= "-" . $3. "-" . $4;

                    $d_ref->{BSC_Id} = $1;
                    $d_ref->{AdjCell} = $5;

                    # Handling AdjCell # in hexa for block M536870935
                    my $myAdjCell_Hexa = substr( $5, -4 );
                    $d_ref->{AdjCell} = hex($myAdjCell_Hexa);
                }
                else {
                    LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
                }
            }
            ## Case 2: HBSC_1_Pleven/Cell:VT5251-3/AdjCell:VT5048-1
            ## CR valnt00016762, 09 Apr 2008
            ## AdjCell should be without prefix characters (cf. AdjCell 50481)
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)-(\d+)\/AdjCell:(\w+)-(\d+)/ ) {
                ## CR valnt00007899,02 Oct 2007
                ## AdjCell should also be without "-" (cf. AdjCell VT50481)
                $d_ref->{BSC_Id} = $1;
                $d_ref->{Cell}   = $2 . $3;

                ## CR valnt00007899,02 Oct 2007
                ## AdjCell should also be without "-" (cf. AdjCell VT50481)
                $d_ref->{AdjCell} = $4 . $5;
            }
            ## Case 3: HBSC_1_Pleven/Cell:VT5251-3/AdjCell:284033f13ecc667 (hex)
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)-(\d+)\/AdjCell:(.+)/ ) {
                $d_ref->{BSC_Id} = $1;
                $d_ref->{Cell}   = $2 . $3;

                # Handling AdjCell # in hexa for block M536870935
                my $myAdjCell_Hexa = substr( $4, -4 );
                $d_ref->{AdjCell} = hex($myAdjCell_Hexa);
            }
            ## Case 4: BMDN2/Cell:MC502_SBaruMedan/AdjCell:3070_Aurmedan-3
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)\/AdjCell:(.+)-(\d+)/ ) {
                $d_ref->{BSC_Id}  = $1;
                $d_ref->{Cell}    = $2;
                $d_ref->{AdjCell} = $3 . $4;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : Processor (txt)
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GSM/BSC Data - Processor',
        INPUT_FILE_DESCRIPTION     => ['^pmresult_536870917_\d{2}_\d{12}_\d{12}.txt$'],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => '\t',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).txt',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.txt$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.txt$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.txt$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.txt$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("Processor: BSC: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the BSC_Id, and Module from format <BSC_Id>/Module:<Module>
            if ( $d_ref->{Object_Name} =~ /(.+)\/Module:(.+)/ ) {
                $d_ref->{BSC_Id} = $1;
                $d_ref->{Module} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : Signalling_Link (txt)
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GSM/BSC Data - Signalling_Link',
        INPUT_FILE_DESCRIPTION     => ['^pmresult_536870921_\d{2}_\d{12}_\d{12}.txt$'],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => '\t',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).txt',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.txt$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.txt$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.txt$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.txt$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("Signalling Link: BSC: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the BSC_Id, SignLink, SignLinkset from format <BSC_Id>/SignLink:<SignLink>
            ## Case 1: HBSC_1_Pleven/SignLink:MSC-2-8
            if ( $d_ref->{Object_Name} =~ /(.+)\/SignLink:(\w+-\w+)(-\w+)/ ) {
                $d_ref->{BSC_Id}      = $1;
                $d_ref->{SignLink}    = $2 . $3;
                $d_ref->{SignLinkset} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : TC_Board (txt)
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GSM/BSC Data - TC_Board',
        INPUT_FILE_DESCRIPTION     => [
            '^pmresult_536870957_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870970_\d{2}_\d{12}_\d{12}.txt$'
        ],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => '\t',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).txt',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.txt$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.txt$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.txt$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.txt$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("TC_BOARD: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq "" || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the BSC_Id, Module, Frame, Board from format <BSC_Id>/Module:<Module>/Frame:<Frame>/Board:<Board>
            if ( $d_ref->{Object_Name} =~ /(.+)\/Module:(.+)\/Frame:(.+)\/Board:(.+)/ ) {
                $d_ref->{BSC_Id} = $1;
                $d_ref->{Module} = $2;
                $d_ref->{Frame}  = $3;
                $d_ref->{Board}  = $4;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GSM Files (with counter name) : TRX (txt)
    #-----------------------------------------
    {
        RULE_TYPE              => 'GENERIC_CSV_GATEWAY',
        RULE_DESC              => 'Generic CSV Gateway For GSM/BSC Data - TRX',
        INPUT_FILE_DESCRIPTION => [
            '^pmresult_526870942_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870938_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870939_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870940_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870941_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870942_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870944_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870945_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870946_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870953_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870954_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870955_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870956_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_536870969_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_536870973_\d{2}_\d{12}_\d{12}.txt$'
        ],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => '\t',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).txt',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.txt$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.txt$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.txt$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.txt$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;

            #LogMess("TRX: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "BSC_", $block_name);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the BSC_Id, Cell, Trx
            ## Case 1: BSCSB01/Cell:Label=ST067GC,_MODUNO=8,_CELLLNUM=9/Trx:TRX-11
            if ( $d_ref->{Object_Name} =~ /(.+)\/Cell:Label=(.+),_?MODUNO=(\d+),_?CELLLNUM=(\d+)\/Trx:(.+)/ ) {

                # Remove hyphen then append MODUNO and CELLNUM
                $d_ref->{Cell}  = $2;
                $d_ref->{Cell}  =~ s/-//g;
                $d_ref->{Cell} .= "-" . $3 . "-" . $4;

                $d_ref->{BSC_Id} = $1;
                $d_ref->{Trx}  = $5;
            }
            ## Case 2: HBSC_1_Pleven/Cell:VT5378-2/Trx:TRX-4
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)-(.+)\/Trx:(.+)/ ) {

                ## CR valnt00007899,02 Oct 2007
                ## Cell should append the numeric number after the  "-" (cf. Cell:Test_Site-3 to be Cell Test_Site3)
                ## old code: $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)-.+\/Trx:(.+)/;
                ## 26 Oct 2007
                ## Fixing object name with BPKB1/Cell:4402_Nangka-3(BSC)/Trx:TRX-9
                ## \d+ would not match 3(BSC), hence change it to .+
                ## old code: $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)-(\d+)\/Trx:(.+)/;

                $d_ref->{BSC_Id} = $1;

                ## CR valnt00007899,02 Oct 2007
                ## Cell should append the numeric number after the  "-" (cf. Cell:Test_Site-3 to be Cell Test_Site3)
                $d_ref->{Cell} = $2 . $3;
                $d_ref->{Trx}  = $4;
            }
            ## Case 3: BMDN2/Cell:MC502_SBaruMedan/Trx:TRX-1
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/Cell:(.+)\/Trx:(.+)/ ) {

                $d_ref->{BSC_Id} = $1;
                $d_ref->{Cell}   = $2;
                $d_ref->{Trx}    = $3;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GPRS Files (with counter name) - BSC (txt)
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GPRS/PCU DATA - BSC',
        INPUT_FILE_DESCRIPTION     => ['^pmresult_587203160_\d{2}_\d{12}_\d{12}.txt$'],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => '\t',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).txt',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.txt$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.txt$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.txt$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.txt$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;    ## 02 Nov 2007, declare $block_name

            #LogMess("PCU-BSC: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};                     ## 02 Nov 2007
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "PCU_", $block_name);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the PCU_Id, PCUBSC from format <PCU_Id>/PCUBSC:<PCUBSC>
            if ( $d_ref->{Object_Name} =~ /(.+)\/PCUBSC:(.+)/ ) {
                $d_ref->{PCU_Id} = $1;
                $d_ref->{PCUBSC} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GPRS Files (with counter name) - CELL (txt)
    #-----------------------------------------
    {
        RULE_TYPE              => 'GENERIC_CSV_GATEWAY',
        RULE_DESC              => 'Generic CSV Gateway For GPRS/PCU DATA - CELL',
        INPUT_FILE_DESCRIPTION => [
            '^pmresult_587203010_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_587202560_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_587202580_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_587202600_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_587202620_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_587202640_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_587202660_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_587202690_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_587202710_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_587202740_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_587202770_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_587202800_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_587202830_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_587202860_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_587202890_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_587202920_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_587202940_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_587202960_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_587202970_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_587202990_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_587203040_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_587203070_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_587203110_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_587204560_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_587204570_\d{2}_\d{12}_\d{12}.txt$', '^pmresult_587204580_\d{2}_\d{12}_\d{12}.txt$',
            '^pmresult_587204590_\d{2}_\d{12}_\d{12}.txt$'
        ],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => '\t',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).txt',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.txt$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.txt$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.txt$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.txt$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;    ## 02 Nov 2007, declare $block_name

            #LogMess("PCU-Cell: d_ref -> \n" . Dumper($d_ref), 5);


            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "PCU_", $block_name);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {
                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;
            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};
            }


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the PCU_Id, PCUCell, PCUCell_name, BTSSiteManager
            ## Case 1: PCU2/PCUCell:syl_x0043_1
            if ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(\w+)_(\D+\d+)_(\d+)/ ) {

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $3 . $4;
                $d_ref->{PCUCell_name}   = $2 . "_" . $3 . "_" . $4;
                $d_ref->{BTSSiteManager} = $3;

            }
            ## Case 2: PCUMKS1/PCUCell:3747_antang-1 or PCUPTK1/PCUCell:3408+sambas-1 or PCUMKS1/PCUCell:3747antang-1
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(\d{4})[-+_]?(.+)-(\d{1})/ ) {

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $2 . $4;
                $d_ref->{PCUCell_name}   = $3 . "-" . $4;
                $d_ref->{BTSSiteManager} = $2;

            }
            ## Case 3: PCUMKS1/PCUCell:37463_sengkang or PCUBLP1/PCUCell:39712kuriang
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(\d{4})(\d{1})[-+_]?(.+)/ ) {

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $2 . $3;
                $d_ref->{PCUCell_name}   = $4 . "-" . $3;
                $d_ref->{BTSSiteManager} = $2;

            }
            ## Case 4: PCUMKS1/PCUCell:pc549_mtcmakassar-1
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(\w+)[-+_](\w+)-(\d+)/ ) {

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $2 . $4;
                $d_ref->{PCUCell_name}   = $3 . "-" . $4;
                $d_ref->{BTSSiteManager} = $2;

            }
            ## Case 5: PCUMKS1/PCUCell:pc197_gtc_tj_bunga
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(.+\d{1})[-+_](.+)/ ) {

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $2;
                $d_ref->{PCUCell_name}   = $3;
                $d_ref->{BTSSiteManager} = $2;

            }
            ## Case 6: PCU2/PCUCell:vn45423=huawei_kaolinovo
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(.+)(\d{1})=(.+)/ ){

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $2 . $3;
                $d_ref->{PCUCell_name}   = $4 . "-" . $3;
                $d_ref->{BTSSiteManager} = $2;

            }
            ## Case 7: PCU4/PCUCell:tbtshu_1 or PCU_URA/PCUCell:urfeg-1
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(\w+)[_-](\d+)/ ) {

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $2 . $3;
                $d_ref->{PCUCell_name}   = $2 . $3;
                $d_ref->{BTSSiteManager} = $2;

            }
            ## Case 8: PCU1/PCUCell:test
            elsif ( $d_ref->{Object_Name} =~ /(.+)\/PCUCell:(.+)/ ) {

                $d_ref->{PCU_Id}         = $1;
                $d_ref->{PCUCell}        = $2;
                $d_ref->{PCUCell_name}   = $2;
                $d_ref->{BTSSiteManager} = $2;

            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


						if ( !defined($d_ref->{PCU_Id}) or $d_ref->{PCU_Id} eq "" or
						     !defined($d_ref->{PCUCell}) or $d_ref->{PCUCell} eq "" or
						     !defined($d_ref->{PCUCell_name}) or $d_ref->{PCUCell_name} eq "" or
						     !defined($d_ref->{BTSSiteManager}) or $d_ref->{BTSSiteManager} eq "" ) {
                LogMess("Invalid objects for PCU DATA - CELL!", 5);
            }


            # Ignore record if PCU Id is not valid
            if ( $d_ref->{PCU_Id} =~ /^[\s_\/\\]$/ ) {
                LogMess( "{PCU_Id} is equal to blank or underscore. The related block is removed." );
                $$blkname_ref = "";
                return;
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }


            return;
        }
    },

    #-----------------------------------------
    # Huawei BSS GPRS Files (with counter name) - LAPD (txt)
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GPRS/PCU DATA - LAPD',
        INPUT_FILE_DESCRIPTION     => ['^pmresult_587203180_\d{2}_\d{12}_\d{12}.txt$'],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => '\t',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).txt',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.txt$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.txt$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.txt$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.txt$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;    ## 02 Nov 2007, declare $block_name

            #LogMess("PCU-LAPD: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};                     ## 02 Nov 2007
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "PCU_", $block_name);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the PCU_Id, PCULAPD from format <PCU_Id>/PCULAPD:<PCULAPD>
            if ( $d_ref->{Object_Name} =~ /(.+)\/PCULAPD:(.+)/ ) {
                $d_ref->{PCU_Id}  = $1;
                $d_ref->{PCULAPD} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GPRS Files (with counter name) - NSVC (txt)
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GPRS/PCU DATA - NSVC',
        INPUT_FILE_DESCRIPTION     => ['^pmresult_587203210_\d{2}_\d{12}_\d{12}.txt$'],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => '\t',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).txt',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.txt$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.txt$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.txt$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.txt$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },
        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;    ## 02 Nov 2007, declare $block_name

            #LogMess("PCU-NSVC: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};                     ## 02 Nov 2007
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "PCU_", $block_name);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the PCU_Id, PCULAPD from format <PCU_Id>/PCUNSVC:<PCUNSVC>
            if ( $d_ref->{Object_Name} =~ /(.+)\/PCUNSVC:(.+)/ ) {
                $d_ref->{PCU_Id}  = $1;
                $d_ref->{PCUNSVC} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },

    #-----------------------------------------
    # Huawei BSS GPRS Files (with counter name) - Processor (txt)
    #-----------------------------------------
    {
        RULE_TYPE                  => 'GENERIC_CSV_GATEWAY',
        RULE_DESC                  => 'Generic CSV Gateway For GPRS/PCU DATA - Processor',
        INPUT_FILE_DESCRIPTION     => ['^pmresult_587203240_\d{2}_\d{12}_\d{12}.txt$'],
        INPUT_DIR_DEPTH            => 3,
        NUMBER_OF_FILES_TO_PROCESS => 1000,
        ORDER_OF_FILES             => 'OLDEST_FIRST',

        # The following are specific to GSM Huawei BSS GSM V300R002 raw files
        # ---------------------------------------------------------------
        ## 7 Sept 2007 - Bugs: 57963, 57978
        ## Description:
        ##    - increase the range to allow more fields/counters to be read
        ##    - largest count of counters from http://notesdoc/87256D28003200ED/0/96F24CC77A5A93ED802572FB000C118E/$file/Performance+Data(BSC).xls is 96
        DATA_FIELDS => [ 0 .. 99 ],

        INVALID_VALUE_MATCH => [],
        FIELD_SEPARATOR     => '\t',
        WHITESPACE_MODIFIER => '_',
        PRE_PARSE           => 'HUAWEI_BSS_GSM_NAME',

        #ENABLE_FILENAME_TIMESTAMP    => 'True',
        PIF_FILENAME_EXTRACT => '(.*).txt',

        ## Added 27 Aug 2007
        ## added to get the region_id & network_id from raw file
        DIRECTORY_HEADER_FIELDS => {
            NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },

        FILENAME_HEADER_FIELDS => {
            blkName   => '^pmresult_(\d{9})_\d{2}_\d{12}_\d{12}.txt$',
            filename  => '(^pmresult_\d{9}_\d{2}_\d{12}_\d{12}.txt$)',
            startDate => '^pmresult_\d{9}_\d{2}_(\d{8})\d{4}_\d{12}.txt$',
            endDate   => '^pmresult_\d{9}_\d{2}_\d{12}_(\d{8})\d{4}.txt$',
			Dname     => '^pmresult_\d{9}_\d{2}_(\d{12}_\d{12}).(csv|txt)$', #IZ67841
        },

        HEADER_DATA_RECORD_PROCESSING => sub {
            my ( $blkname_ref, $h_ref, $d_ref ) = @_;
            my $block_name;    ## 02 Nov 2007, declare $block_name

            #LogMess("PCU-Processor: d_ref -> \n" . Dumper($d_ref), 5);


            # named the pif block according to the Function SubsetID
            if ( exists( $h_ref->{blkName} ) ) {
                $$blkname_ref = "M" . $h_ref->{blkName};
                $block_name   = "M" . $h_ref->{blkName};                     ## 02 Nov 2007
            }

            # Add the first time instance for DATE to header if not exists
            $h_ref->{DATE} = "0000-00-00" unless exists( $h_ref->{DATE} );


            # Process the counter name and data
            process_counters($d_ref, "PCU_", $block_name);


            ########### [START] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)
            if ( $d_ref->{Object_Name} eq "_" || $d_ref->{Object_Name} eq ""  || $d_ref->{Object_Name} eq " ") {

                LogMess( "{Object_Name} is equal to blank or underscore. The related block is removed." );

                #foreach $newkeys ( keys %$d_ref ) {
                #    $d_ref->{$newkeys} = "";
                #}
                $$blkname_ref = "";
                return;

            }
            ########### [END] GSM VN: remove block with Object_Name = underscore (_) (blank Object_name)


            # Extract date and time
            get_date_time($h_ref, $d_ref);


            # Extracting the PCU_Id, PCUBoard from format <PCU_Id>/PCUBoard:<PCUBoard>
            if ( $d_ref->{Object_Name} =~ /(.+)\/PCUBoard:(.+)/ ) {
                $d_ref->{PCU_Id}   = $1;
                $d_ref->{PCUBoard} = $2;
            }
            else {
                LogMess( "WARNING: Received an unrecognized Object_name: [$d_ref->{Object_Name}] ", 5 );
            }


            # Renaming Granularity_Period to GP
            if ( exists $d_ref->{Granularity_Period} ) {
                $d_ref->{GP} = $d_ref->{Granularity_Period};
                delete $d_ref->{Granularity_Period};
            }

            return;
          }
    },
);

sub engine_config {
    return \@rules;
}

1;
